
from zeam.form.base.fields import Fields
from zeam.form.base.form import FormCanvas
from zeam.form.base.form import StandaloneForm, cloneFormData
from zeam.form.base.widgets import Widgets, getWidgetExtractor
from zeam.form.composed.form import SubFormBase

from zeam.form.table.select import SelectField
from zeam.form.table.actions import TableActions
from zeam.form.table import interfaces
from zeam.utils.batch import Batch, IBatching

from zope.component import queryMultiAdapter
from grokcore import component as grok
from megrok import pagetemplate as pt
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('zeam.form.base')


pt.templatedir('default_templates')


class TableFormCanvas(FormCanvas):
    """A form which is able to edit more than one content as a table.
    """
    grok.baseclass()
    grok.implements(interfaces.ITableFormCanvas)

    batchSize = 0
    batchFactory = Batch
    batchItemFactory = lambda f, x: x
    tableFields = Fields()
    tableActions = TableActions()
    emptyDescription = _(u"There are no items.")

    def __init__(self, context, request):
        super(TableFormCanvas, self).__init__(context, request)
        self.lines = []
        self.lineWidgets = []
        self.batching = None

    def createSelectedField(self, item):
        """Return a field to select the line.
        """
        return SelectField(identifier='select')

    def updateLines(self, mark_selected=False):
        self.lines = []
        self.lineWidgets = []
        self.batching = None
        items = self.getItems()
        if self.batchSize:
            items = self.batchFactory(
                items,
                name=self.prefix,
                factory=self.batchItemFactory,
                count=self.batchSize,
                request=self.request)
            self.batching = queryMultiAdapter(
                (self.getFormForTable(), items, self.request), IBatching)()
        for position, item in enumerate(items):
            prefix = '%s.line-%d' % (self.prefix, position)
            form = cloneFormData(self, content=item, prefix=prefix)
            form.selected = False

            # Checkbox to select the line
            form.selectedField = self.createSelectedField(item)

            if mark_selected:
                # Mark selected lines
                selectedExtractor = getWidgetExtractor(
                    form.selectedField, form, self.request)
                if selectedExtractor is not None:
                    value, error = selectedExtractor.extract()
                    if value:
                        form.selected = True

            lineWidget = Widgets(form=form, request=self.request)
            lineWidget.extend(form.selectedField)
            self.lines.append(form)
            self.lineWidgets.append(lineWidget)

    def updateActions(self):
        form, action, status = self.tableActions.process(self, self.request)
        if action is None:
            form, action, status = self.actions.process(self, self.request)
        return form, action, status

    def updateWidgets(self):
        self.updateLines()
        for widgets in self.lineWidgets:
            widgets.extend(self.tableFields)
        self.fieldWidgets.extend(self.fields)
        self.actionWidgets.extend(self.tableActions)
        self.actionWidgets.extend(self.actions)

        for widgets in self.lineWidgets:
            widgets.update()
        self.fieldWidgets.update()
        self.actionWidgets.update()

    def getItems(self):
        return self.context.values()


class TableForm(TableFormCanvas, StandaloneForm):
    """A full standalone TableForm.
    """
    grok.baseclass()
    grok.implements(interfaces.ITableForm)

    def getFormForTable(self):
        return self


class SubTableForm(SubFormBase, TableFormCanvas):
    """A table form which can be used in a composed form.
    """
    grok.baseclass()
    grok.implements(interfaces.ISubTableForm)

    def getFormForTable(self):
        return self.getComposedForm()

class SubTableFormTemplate(pt.PageTemplate):
    """A default template for a SubTableForm
    """
    pt.view(SubTableForm)




from zeam.form.base import *
from zeam.form.table.actions import TableActions
from zeam.form.table.actions import TableSelectionActions
from zeam.form.table.actions import TableMultiActions
from zeam.form.table.form import TableForm, SubTableForm

from zeam.form.table.interfaces import IZeamFormTableAPI
__all__ = list(IZeamFormTableAPI)




from zeam.form.base.fields import Field
from zeam.form.base.widgets import FieldWidget, WidgetExtractor

from grokcore import component as grok


class SelectField(Field):
    # This field is always in input and have a different prefix
    mode = 'input'
    prefix = 'select'
    ignoreContent = True
    ignoreRequest = False
    defaultValue = False


class SelectFieldWidget(FieldWidget):
    grok.adapts(SelectField, None, None)

    def htmlClass(self):
        cls = ['field', '-'.join([self.form.parent.htmlId(), 'select'])]
        return ' '.join(cls)

    def prepareContentValue(self, value):
        return {self.identifier: bool(value)}


class SelectFieldExtractor(WidgetExtractor):
    grok.adapts(SelectField, None, None)

    def extract(self):
        value, error = WidgetExtractor.extract(self)
        if value == 'selected':
            return True, None
        return False, None




from zope import interface

from zeam.form.base.interfaces import IFormCanvas, IForm
from zeam.form.base.interfaces import IZeamFormBaseAPI
from zeam.form.composed.interfaces import ISubForm


class ITableFormCanvas(IFormCanvas):
    """Base form behavior for forms working on more than one content
    at a time.
    """

    lines = interface.Attribute(u"Widgets lines")

    def updateLines():
        """Prepare widgets and forms for each lines.
        """

    def getItems():
        """Return the list of contents.
        """


class ITableForm(IForm, ITableFormCanvas):
    """A form which is able to work on more than one item at a time.
    """


class ISubTableForm(ISubForm, ITableFormCanvas):
    """A table form that can be used in a composed form.
    """


class IZeamFormTableAPI(IZeamFormBaseAPI):
    """Public API of zeam.form.table
    """

    TableForm = interface.Attribute(
        u"A form able to work on more on more than one item at a time")
    SubTableForm = interface.Attribute(
        u"A table form that can be used in a composed form")
    TableActions = interface.Attribute(
        u"Action being executed on more than one content at a time")
    TableSelectionActions = interface.Attribute(
        u"Action executed on the selection of content")
    TableMultiActions = interface.Attribute(
        u"Action excted one time for multiple content")





from zeam.form.base.markers import NO_VALUE, SUCCESS, FAILURE, NOTHING_DONE
from zeam.form.base.markers import getValue
from zeam.form.base.actions import Actions
from zeam.form.base.errors import Error
from zeam.form.base.interfaces import IWidgetExtractor, ActionError

from zeam.form.table import interfaces

from zope import component
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('zeam.form.base')


class TableActions(Actions):
    """Actions that can be applied on a table line.

    Actions are applied for each selected table line.
    """

    def process(self, form, request):
        assert interfaces.ITableFormCanvas.providedBy(form)
        one_selected = False
        ready = False

        for action in self:
            isPostOnly = getValue(action, 'postOnly', form)
            if isPostOnly and request.method != 'POST':
                form.errors.append(
                    Error('This form was not submitted properly',
                          form.prefix))
                return form, None, FAILURE
            extractor = component.getMultiAdapter(
                (action, form, request), IWidgetExtractor)
            value, error = extractor.extract()
            if value is not NO_VALUE:
                if not ready:
                    form.updateLines(mark_selected=True)
                    ready = True
                for line in form.lines:
                    if not line.selected:
                        continue
                    one_selected = True
                    try:
                        if action.validate(form):
                            content = line.getContentData().getContent()
                            action(form, content, line)
                    except ActionError, e:
                        line.errors.append(Error(e.args[0], line.prefix))
                if not one_selected:
                    form.errors.append(
                        Error(_(u"You didn't select any item!"), None))
                    return form, action, FAILURE
                return form, action, SUCCESS
        return form, None, NOTHING_DONE


class TableSelectionActions(Actions):
    """Actions that manage a selection of table lines.

    Actions are applied only one time with a list of changes regarding
    which lines have been selected and unselected from the original
    content selection.
    """

    def process(self, form, request):
        assert interfaces.ITableFormCanvas.providedBy(form)
        selected_lines = []
        deselected_lines = []
        unchanged_lines = []

        # mark selected by request
        form.updateLines(mark_selected=True)
        for line in form.lines:
            data = line.getContentData()
            content_selected = data.get(line.selectedField.identifier)
            if content_selected:
                if line.selected:
                    unchanged_lines.append(line)
                else:
                    deselected_lines.append(line)
            else:
                if line.selected:
                    selected_lines.append(line)
                else:
                    unchanged_lines.append(line)

        status = NOTHING_DONE
        for action in self:
            isPostOnly = getValue(action, 'postOnly', form)
            if isPostOnly and request.method != 'POST':
                form.errors.append(
                    Error('This form was not submitted properly',
                          form.prefix))
                return form, None, FAILURE

            extractor = component.getMultiAdapter(
                (action, form, request), IWidgetExtractor)
            value, error = extractor.extract()
            if value is NO_VALUE:
                continue

            try:
                if action.validate(form):
                    return form, action, action(
                        form,
                        selected_lines,
                        deselected_lines,
                        unchanged_lines)
            except ActionError, e:
                form.errors.append(Error(e.args[0], form.prefix))
                return form, action, FAILURE

        return form, None, status


class TableMultiActions(Actions):
    """Actions that manage multiple table lines.

    Actions are applied one time with a list of currently selected and
    unselected lines.
    """

    def process(self, form, request):
        assert interfaces.ITableFormCanvas.providedBy(form)
        selected_lines = []
        unselected_lines = []
        ready = False

        for action in self:
            isPostOnly = getValue(action, 'postOnly', form)
            if isPostOnly and request.method != 'POST':
                form.errors.append(
                    Error('This form was not submitted properly',
                          form.prefix))
                return form, None, FAILURE
            extractor = component.getMultiAdapter(
                (action, form, request), IWidgetExtractor)
            value, error = extractor.extract()
            if value is not NO_VALUE:
                if not ready:
                    form.updateLines(mark_selected=True)
                    for line in form.lines:
                        if line.selected:
                            selected_lines.append(line)
                        else:
                            unselected_lines.append(line)
                    ready = True

                try:
                    if action.validate(form):
                        return form, action, action(
                            form,
                            selected_lines,
                            unselected_lines)
                except ActionError, e:
                    form.errors.append(Error(e.args[0], form.prefix))
                    return form, action, FAILURE

        return form, None, NOTHING_DONE

